package com.gmei.up

import java.util.Properties
import org.apache.flink.api.scala._
import org.apache.flink.streaming.api.scala.{ StreamExecutionEnvironment, DataStream }
import org.apache.flink.streaming.connectors.kafka.{
  FlinkKafkaConsumer,
  FlinkKafkaProducer,
  KafkaDeserializationSchema
}
import org.apache.flink.api.common.typeinfo.{ TypeHint, TypeInformation, Types }
import org.apache.kafka.clients.consumer.ConsumerRecord
import com.alibaba.fastjson.JSON

case class KafkaUserInfo(
    device_id: String,
    action: String,
    log_time: Double,
    event_cn: String,
    second_demands: Array[String],
    projects: Array[String]
)

class UserInfoDeserializationSchema extends KafkaDeserializationSchema[KafkaUserInfo] {
  override def isEndOfStream(t: KafkaUserInfo): Boolean = false

  override def deserialize(consumerRecord: ConsumerRecord[Array[Byte], Array[Byte]]): KafkaUserInfo = {
    val info = new String(consumerRecord.value(), "UTF-8")
    val userInfo: KafkaUserInfo = JSON.parseObject(info, classOf[KafkaUserInfo])
    userInfo
  }

  override def getProducedType: TypeInformation[KafkaUserInfo] =
    Types.GENERIC(classOf[KafkaUserInfo])
}

object Main {
  def main(args: Array[String]): Unit = {
    val env = StreamExecutionEnvironment.getExecutionEnvironment

    // TODO read from config
    val kafkaConsumerProperties = new Properties
    kafkaConsumerProperties.setProperty("group.id", "user_portrait_flink_streaming")
    kafkaConsumerProperties.setProperty(
      "bootstrap.servers",
      "172.16.44.25:9092,172.16.44.31:9092,172.16.44.45:9092"
    )

    val kafkaConsumer = new FlinkKafkaConsumer[KafkaUserInfo](
      "gm-portrait-update-device",
      new UserInfoDeserializationSchema,
      kafkaConsumerProperties
    )

    val stream: DataStream[KafkaUserInfo] = env.addSource(kafkaConsumer)

    stream.map { user =>
      println(user.device_id)
      user.second_demands.foreach(println)
      println("########################")
    }

    stream.print
    env.execute("flink streaming user portrait")
  }
}
